@@ -67,6 +67,7 @@ Iterate over tokens in a (part of a) Document, with or without state.
 from __future__ import unicode_literals
 from __future__ import absolute_import
 
+import io
 import sys
 import operator
 import collections
@@ -117,12 +118,13 @@ class DocumentBase(object):
         """Constructor"""
         self._writing = 0
         self._changes = collections.defaultdict(list)
-        # to keep compatible with 2.6 (else we'd use a WeakSet)
-        self._cursors = weakref.WeakKeyDictionary()
+        self._cursors = weakref.WeakSet()
     
-    def __nonzero__(self):
+    def __bool__(self):
         return True
     
+    __nonzero__ = __bool__      # py2 compat
+    
     def __iter__(self):
         """Iter over all blocks."""
         return self.blocks_forward(self[0])
@@ -236,7 +238,7 @@ class DocumentBase(object):
         The Cursor gets updated when the document is changed.
         
         """
-        self._cursors[cursor] = True
+        self._cursors.add(cursor)
         
     def check_changes(self):
         """Debugging method that checks for overlapping edits."""
@@ -344,6 +346,14 @@ class Document(DocumentBase):
         self._guessed_mode = None
         self.setplaintext(text)
     
+    @classmethod
+    def load(cls, filename, encoding='utf-8', mode=None):
+        """Load the document from a file, using the specified encoding and mode."""
+        with io.open(filename, encoding=encoding) as f:
+            doc = cls(f.read(), mode)
+        doc.filename = filename
+        return doc
+    
     def __len__(self):
         """Return the number of blocks"""
         return len(self._blocks)
@@ -701,6 +711,13 @@ class Runner(object):
             self._index = len(self._tokens) if at_end else -1
             return True
     
+    def _newline(self):
+        """(Internal) Create a Newline token at the end of the current block."""
+        pos = len(self._doc.text(self.block))
+        if self._wp:
+            pos += self._doc.position(self.block)
+        return ly.lex.Newline('\n', pos)
+        
     def forward_line(self):
         """Yields tokens in forward direction in the current block."""
         end = len(self._tokens)
@@ -716,9 +733,11 @@ class Runner(object):
         while True:
             for t in self.forward_line():
                 yield t
+            newline = self._newline()
             if not self.next_block():
                 break
-    
+            yield newline
+
     def backward_line(self):
         """Yields tokens in backward direction in the current block."""
         if self._index >= 0:
@@ -735,6 +754,7 @@ class Runner(object):
                 yield t
             if not self.previous_block():
                 break
+            yield self._newline()
     
     def previous_block(self, at_end=True):
         """Go to the previous block, positioning the cursor at the end by default.
@@ -819,6 +839,8 @@ class Source(object):
         used.
         
         """
+        self._pushback = False
+        self._last = None
         self._doc = document = cursor.document
         start_block = document.block(cursor.start)
         self._wp = tokens_with_position
@@ -897,6 +919,15 @@ class Source(object):
                     source = token_source
         gen = generator()
         
+        if tokens_with_position:
+            def newline():
+                pos = document.position(self.block) - 1
+                return ly.lex.Newline('\n', pos)
+        else:
+            def newline():
+                pos = len(document.text(document.previous_block(self.block)))
+                return ly.lex.Newline('\n', pos)
+        
         # initialize block and tokens
         for self.block, self.tokens in gen:
             break
@@ -905,18 +936,37 @@ class Source(object):
             for t in self.tokens:
                 yield t
             for self.block, self.tokens in gen:
+                yield newline()
                 for t in self.tokens:
                     yield t
         self._gen = g()
     
     def __iter__(self):
-        return self._gen
+        return self
     
     def __next__(self):
-        return self._gen.next()
+        if self._pushback:
+            self._pushback = False
+            return self._last
+        i = self._last = next(self._gen)
+        return i
     
     next = __next__
     
+    def pushback(self, pushback=True):
+        """Yields the last yielded token again on the next request.
+        
+        This can be called multiple times, but only the last token will be 
+        yielded again. You can also undo a call to pushback() using 
+        pushback(False).
+        
+        """
+        self._pushback = pushback
+    
+    def token(self):
+        """Re-returns the last yielded token."""
+        return self._last
+    
     @property
     def document(self):
         """Return our Document."""
@@ -936,6 +986,18 @@ class Source(object):
             pos += self._doc.position(self.block)
         return pos
     
+    def until_parser_end(self):
+        """Yield the tokens until the current parser is quit.
+        
+        You can only use this method if you have a State enabled.
+        
+        """
+        depth = self.state.depth()
+        for t in self:
+            yield t
+            if self.state.depth() < depth and not self._pushback:
+                break
+        
     def consume(self, iterable, position):
         """Consumes iterable (supposed to be reading from us) until position.
         
