@@ -38,6 +38,11 @@ Note: elements keep a weak reference to their parent.
 from __future__ import unicode_literals
 from __future__ import absolute_import # prevent picking old stale node.py from package
 
+try:
+    string_types = basestring
+except NameError:
+    string_types = str
+
 import fractions
 import re
 
@@ -132,6 +137,13 @@ class Printer(object):
     Performs certain operations on behalf of a LyNode tree,
     like quoting strings or translating pitch names, etc.
     """
+    
+    # You may change these
+    primary_quote_left = '\u2018'
+    primary_quote_right = '\u2019'
+    secondary_quote_left = '\u201C'
+    secondary_quote_right = '\u201D'
+    
     def __init__(self):
         self.typographicalQuotes = True
         self.language = "nederlands"
@@ -139,8 +151,8 @@ class Printer(object):
         
     def quoteString(self, text):
         if self.typographicalQuotes:
-            text = re.sub(r'"(.*?)"', '\u201C\\1\u201D', text)
-            text = re.sub(r"'(.*?)'", '\u2018\\1\u2019', text)
+            text = re.sub(r'"(.*?)"', self.primary_quote_left + r'\1' + self.primary_quote_right, text)
+            text = re.sub(r"'(.*?)'", self.secondary_quote_left + r'\1' + self.secondary_quote_right, text)
             text = text.replace("'", '\u2018')
         # escape regular double quotes
         text = text.replace('"', '\\"')
@@ -177,20 +189,20 @@ class Reference(object):
     def __init__(self, name=""):
         self.name = name
     
-    def __unicode__(self):
+    def __format__(self, format_spec):
         return self.name
 
 
 class Named(object):
     """
     Mixin to print a \\name before the contents of the container.
-    unicode() is called on the self.name attribute, so it may also
+    format() is called on the self.name attribute, so it may also
     be a Reference.
     """
     name = ""
     
     def ly(self, printer):
-        return "\\{0} {1}".format(unicode(self.name), super(Named, self).ly(printer))
+        return "\\{0} {1}".format(self.name, super(Named, self).ly(printer))
         
         
 class HandleVars(object):
@@ -212,10 +224,10 @@ class HandleVars(object):
         Otherwise the same method from the super class is called.
         """
         def newfunc(obj, name, *args):
-            if isinstance(name, basestring):
+            if isinstance(name, string_types):
                 return func(obj, name, *args)
             else:
-                f = getattr(super(HandleVars, obj), func.func_name)
+                f = getattr(super(HandleVars, obj), func.__name__)
                 return f(name, *args)
         return newfunc
 
@@ -287,8 +299,6 @@ class Text(Leaf):
     """ A leaf node with arbitrary text """
     def __init__(self, text="", parent=None):
         super(Text, self).__init__(parent)
-        if not isinstance(text, basestring):
-            text = unicode(text)
         self.text = text
     
     def ly(self, printer):
@@ -397,8 +407,7 @@ class Assignment(Container):
             return self[0]
 
     def ly(self, printer):
-        return "{0} = {1}".format(
-            unicode(self.name), super(Assignment, self).ly(printer))
+        return "{0} = {1}".format(self.name, super(Assignment, self).ly(printer))
 
 
 HandleVars.childClass = Assignment
@@ -416,7 +425,7 @@ class Identifier(Leaf):
         self.name = name
         
     def ly(self, printer):
-        return "\\" + unicode(self.name)
+        return "\\{0}".format(self.name)
 
 
 class Statement(Named, Container):
@@ -584,7 +593,7 @@ class ContextType(Container):
         res.append(self.ctype or self.__class__.__name__)
         if self.cid:
             res.append("=")
-            res.append(printer.quoteString(unicode(self.cid)))
+            res.append(printer.quoteString(format(self.cid)))
         res.append(super(ContextType, self).ly(printer))
         return " ".join(res)
         
@@ -715,7 +724,7 @@ class LyricsTo(LyricMode):
     
     def ly(self, printer):
         res = ["\\" + self.name]
-        res.append(printer.quoteString(unicode(self.cid)))
+        res.append(printer.quoteString(format(self.cid)))
         res.append(super(Named, self).ly(printer))
         return " ".join(res)
         
@@ -845,6 +854,8 @@ class Tempo(Container):
     A tempo setting, like: \\tempo 4 = 100
     May have a child markup or quoted string.
     """
+    before, after = 1, 1
+    
     def __init__(self, duration, value, parent=None):
         super(Tempo, self).__init__(parent)
         self.duration = duration
